import os

ekhaya = os.path.dirname(__file__)

# Path to base images
BASE_PATH = "{:s}/base".format(ekhaya)

# Path to executor images
CAB_PATH = "{:s}/cab".format(ekhaya)

# Path to config templates
CONFIG_TEMPLATES = "{:s}/configs".format(ekhaya)
